import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  curveBasis
} from 'd3';

let svg = select('svg');

let width = +svg.attr('width');
let height = +svg.attr('height');

csv('https://vizhub.com/curran/datasets/temperature-in-san-francisco.csv')
  .then(data => {
    data.forEach(d => {
      d.temperature = +d.temperature;
      d.timestamp = new Date(d.timestamp);
    });
    render(data);
});

let render = data => {
  console.log(data);
  let title = 'A Week in San Francisco';
  
  let xValue = d => d.timestamp;
  
  let yValue = d => d.temperature;
  let yAxisLabel = 'Temperature';
  
  let margin = { top: 60, right: 40, bottom: 88, left: 105 };
  let innerWidth = width - margin.left - margin.right;
  let innerHeight = height - margin.top - margin.bottom;
  
  let xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  let yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
  
  let g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  let xAxis = axisBottom(xScale);
  
  let yAxis = axisLeft(yScale);
  
  let yAxisG = g.append('g').call(yAxis);
  
  let xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  let lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)))
    .curve(curveBasis);
  
  g.append('path')
      .attr('class', 'line-path')
      .attr('d', lineGenerator(data));
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
    
  let upperToolTip = g.append("g")
  	.attr("class", "upperToolTip");
  
  let bottomToolTip =  g.append("g")
  	.attr("class", "bottomToolTip")
    .attr("display","none");

  upperToolTip
  		.attr("display","none")
  		.attr("class","upperToolTip");
  
  let path = upperToolTip.append("path")
  		.attr("d", "M350 200 C500 200 550 200 550 275 C550 350 500 350 400 350 C350 375 350 400 350 400 C350 400 350 375 300 350 C200 350         150 350 150 275 C150 200 200 200 350 200")
  		.attr("filter","url(#shadow)");
  
	upperToolTip
			.append("text")
  		.attr("class","upperToolTipText");
  
  bottomToolTip
    	.append("rect")
      .attr("width", 130)
      .attr("height", 40);
  
  bottomToolTip
  	.append("text")
  	.attr("class","bottomToolTipText");
  
	let updateToolTip = (d) =>{
    let months =["Jan","Feb","Mar",
                "Apr","May","Jun",
                "Jul","Aug","Sep",
                "Oct","Nov","Dec"];
    
    let date = `${d.timestamp.getDate()} ${months[d.timestamp.getMonth()]} ${d.timestamp.getFullYear()}`;
    
    upperToolTip
      .attr("display","null")
      .attr("transform", `translate(${xScale(xValue(d))},${yScale(yValue(d))})`)
  	  .select(".upperToolTipText").text(parseInt(yValue(d)));
    
    bottomToolTip
    	.attr("display","null")
    	.attr("transform", `translate(${xScale(xValue(d))},${innerHeight})`)
    	.select(".bottomToolTipText").text(date);
	} 
  
  g.append("rect")
            .attr("class", "overlay")
            .attr("width", innerWidth)
            .attr("height", innerHeight)
            .on("mousemove", mousemove)
  					.on("mouseout", mouseout);
 	
  function mousemove() {
    let bisectDate = d3.bisector(xValue).left;
    var x0 = xScale.invert(d3.mouse(this)[0]),
        i = bisectDate(data, x0, 1),
        d0 = data[i - 1],
        d1 = data[i],
        d = x0 - xValue(d0) > xValue(d1) - x0 ? d1 : d0;

    updateToolTip(d);
  }
  
  function mouseout(){
    upperToolTip.attr("display","none");
    bottomToolTip.attr("display","none");
  }
  
};